#
# This perl will list all defined baselines
#
#
# 10 March 2016 - Luc Christiaens
#
# step1 - get list of all defined baselines via callback baseline_engine: get_monitored_metric and put metid in @tab2
# step2 - get list of qos  
# step3 - match each qos regex filters
# step4 - for the matching qos entries check if we have an existing baseline and if yes: list/delete
#
# Updates:
# 
# 
# input parameters:
# -d: output directory (default: c:\temp)
# -f: output file (default: report_nimsoft_baseline)
#
# Possible parameters
# - -a: action: l: list or d: delete baseline with matching qos k: kill baselines that don't have a matching qos (default: l)
# - -v: verbose (y,n) default n. Display non matching qos
#
# --- these are possible regex matches (Case Insensitive)
# -q: qos name include regex 
# -g: oriGin include regex (Attention, this will/can delete a lot)
# -o: prObe include regex 
# -b: robot include regex 
# -s: source include regex 
# -t: target include regex 
#
#

# --- import
use Nimbus::API;
use Nimbus::Session;
use Nimbus::CFG;
use Nimbus::PDS;
use Getopt::Std;
use DBI;
use DBD::ODBC;
# crypt
use Crypt::RC4;
use MIME::Base64;
# UIM generic package
use nimsoft_generic;
# - change directory to directory where perl source is located to find our nimsoft_generic.dat file
use Cwd 'chdir';
use File::Basename;
chdir dirname $0;

# --- variables ---
$SERVER=$ENV{"computername"};
@hostnsplit=split(/\\/, $SERVER);
$hostnn=$#hostnsplit;
$hostn = $hostnsplit[$hostnn];
# - crypt key
$crkey       = "ab_def%hij&5";

# --- get parameters
&getparams();

# --- output file with list of baselines
$OUTPUT_FILE="$outfil";

# --- counters
@tab1 = ();
@tab2 = ();
$count9=0;
$sqlcount=0;
$level=-1;
$metcount=0;
$match=0;
$first="y";

# - these are needed for uim requests
$request_timeout = 30;
$request_retries = 3;

# - show filters
print "Active regex filters:\n";
print "  - qos   : $filt_qos\n";
print "  - origin: $filt_origin\n"; 
print "  - probe : $filt_probe\n";
print "  - robot : $filt_robot\n";
print "  - source: $filt_source\n";
print "  - target: $filt_target\n";
print "\n";

# - Open output file
open(OUTCSV, ">$OUTPUT_FILE.txt") || die "Can't open $OUTPUT_FILE: $!\n";

# - connect to sql
&connect_sql;

# --- create uim defined baseline list (@tab2)
&uim_baseline_list;

# - create table of met_id - qos - target (@tab1)
&uim_get_qos;

# --- end
print "Created file: $OUTPUT_FILE.txt\n";

# --- close outfime
close(OUTCSV);
# --- disconnect ---
$dbh->disconnect();

#
# ------ Sub Procedures ------
#

sub connect_sql
{
# --- connect DB ---
if ($sqltype eq 'my')
  {
    $dbh = DBI->connect("DBI:mysql:database=$database;host=$sqlserver", "$sqluser", "$sqlpass") || die "Could not connect to nimsoftslm database on $sqlserver: $DBI::errstr";
  }
if ($sqltype eq 'ms')
  {
    if ($trusted eq 'yes')
      {
        $dbh = DBI->connect("DBI:ODBC:Driver={SQL Server};Server=$sqlserver;Database=$database;Trusted_Connection=Yes;") or die("\n\nCONNECT ERROR:\n\n$DBI::errstr");
      }
    else
      {
        $dbh = DBI->connect("DBI:ODBC:Driver={SQL Server};Server=$sqlserver;Database=$database;UID=$sqluser;PWD=$sqlpass") or die("\n\nCONNECT ERROR:\n\n$DBI::errstr");
      }
  }
}

sub uim_get_qos
{
# --- sql query (in this case mysql and mssql are the same)
$sql = "SELECT qd.source, qd.qos, qd.target, qd.robot, qd.r_table, qd.table_id, qd.ci_metric_id, qd.probe, qd.origin, ci.ci_name, cid.ci_type, cid.ci_description FROM cm_configuration_item_definition cid,cm_configuration_item_metric cim,s_qos_data qd,cm_configuration_item ci,cm_device d,cm_computer_system cs WHERE cs.cs_id = d.cs_id AND d.dev_id = ci.dev_id AND cim.ci_metric_id = qd.ci_metric_id AND ci.ci_type = cid.ci_type AND ci.ci_id = cim.ci_id ORDER BY cid.ci_description, qd.probe, qd.qos, qd.target, qd.source ";
$sth = $dbh->prepare($sql);
$sth->execute or die "SQL Error: $DBI::errstr\n";

while (@row = $sth->fetchrow_array) 
 {
# - decode line
  ($qsource,$qqos,$qtarget,$qrobot,$qrtable,$qtableid,$qmetid,$qprobe,$qorigin,$qci,$qcitype,$cidescription) = @row;
  $domatch=0;
# - match on filters
#   $domatch: 0 => no matching defined
#           : 1 => a filter was matched
#           : 2 => a filter did not match 
# once a filter has a $domatch=2 it will not be evaluated by others
#
    if ($filt_qos ne '')
     {
       if ($qqos =~ /$filt_qos/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }
    if ($filt_origin ne '')
     {
       if ($qorigin =~ /$filt_origin/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }
    if ($filt_probe ne '')
     {
       if ($qprobe =~ /$filt_probe/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }

    if ($filt_robot ne '')
     {
       if ($qrobot =~ /$filt_robot/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }

    if ($filt_source ne '')
     {
       if ($qsource =~ /$filt_source/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }

    if ($filt_target ne '')
     {
       if ($qtarget =~ /$filt_target/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }
# - if there is a match add it in the table (we need at least 1 filter)
    if ($domatch eq '1' || $domatch eq '0')
#    if ($domatch eq '1')
     {
# --- add qos in table (so we can access qos info by metid)
       $tab1{$qmetid}="$qorigin\$qrobot\$qprobe\$qqos\$qsource\$qtarget";
# --- at this point we can check if a baseline exists
       &check_baseline;
       $count9++;
     }
    $sqlcount++;
 }

# how many qos did we see ?
print "step 2 - processed $sqlcount QoS definitions and $count9 records matched your regex\n";
print "step 3 - from the $count9 matching qos entries we found $match baseline definitions\n";

}

sub check_baseline
{
# - we need to check $qmetid in @tab2
#
# --- no we find a matching met_id?
        if( exists($tab2{$qmetid} ) )
         {
           $match++;
           if ($action eq 'l')
            {
              print  OUTCSV "Baseline $match origin: $qorigin robot: $qrobot probe: $qprobe source: $qsource target: $qtarget qos: $qqos\n";
           print "$match: qos: $qqos robot: $qrobot probe: $qprobe source: $qsource target: $qtarget qos_met: $qmetid\n";
            }
           if ($action eq 'd')
            {
# - Generate baseline command in file
             if ($first eq 'y')
              {
                $first="n";
                print OUTCSV "cd C:\\Program Files (x86)\\Nimsoft\\probes\\slm\\baseline_engine\n";
              } 
             print OUTCSV "rem -- qos: $qqos robot: $qrobot probe: $qprobe source: $qsource target: $qtarget \n";
             print OUTCSV "java -cp .;lib/* com.nimsoft.threshold.cmd.BaselineSetter -user $UIM_username -pwd $UIM_password -probe /$UIM_Domain/$UIM_Hub/$UIM_Robot/baseline_engine -o $doaction -id $qmetid\n";
            }
         }
        else
# --- no matching qos entry (anymore)
        {
           if ($verbose eq 'y')
            {
               print "    no_match: qos: $qqos robot: $qrobot probe: $qprobe source: $qsource target: $qtarget qos_met: $qmetid\n";
            }
        } 
}

# ---
sub uim_baseline_list
{
#
# --- Block to read baselines via callback to baseline_engine

$ihub=0;
$robot_count=0;

# --- Login Nimsoft
my $sid = nimLogin($UIM_username, $UIM_password);

# --- get_monitored_metrics
$pds_in = Nimbus::PDS->new();
# print "Callback: get_monitored_metrics on: $baseline_addr\n";
($rc, $pds_data) = doNimNamedRequest($baseline_addr, 'get_monitored_metrics', $pds_in->data(), $request_timeout, $request_retries);
if ($rc)
  {
     print "rc=$rc of nimNamedRequest 'get_monitored_metrics' from hub $hub_addr\n"
  }
$pds_out = Nimbus::PDS->new($pds_data);

# - $pds_out->dump();
doPDS($pds_out);
}

# ---
sub doPDS
{
  my $pds = shift;

  $level++;
  $pds->rewind();
  while(1)
  {
    ($rc, $key, $type, $size, $data) = pdsGetNext($pds->data());
    last if ($rc != NIME_OK);
    $string = $pds_type2string{$type};
#    print("level=$level rc=$rc key=$key type=$string size=$size data=$data\n");
#    print("$level $key $string $size $data\n");
     if ($string eq "PDS_PDS")
      {
        print("$key: $string $size $data\n");
      }
     else
      {
        if ($level eq '1')
         {
           $metcount++;
         }
#        print("   key: $key string: $string size: $size data: $data\n");
        ($i_interval,$i_outliermin,$i_outliermax,$i_nativeinterval,$i_met_id) = split(',', $data);
        ($a_name,$a_met_id) = split(':', $i_met_id);
        $a_met_id=~ tr/"//d;
        $a_met_id=~ tr/}//d;
#        print "$metcount met_id: $a_met_id\n";
# --- add metid in table (so we can use exist to know if a baseline was defined)
        $tab2{$a_met_id}="$a_met_id";

      }
    if ($string eq "PDS_PPDS" || $string eq "PDS_PPCH")
    {
      $pds_data = $pds->get($key, $pds_type{"PDS_PDS"});
      doPDS($pds_data);
    }
    elsif ($string eq "PDS_PDS" )
    {
      $pds_data = Nimbus::PDS->new($data);
      doPDS($pds_data);
    }
  }
  $level--;
if ($level eq '0')
 {
   print "step 1 - processed $metcount baseline definitions\n";
 }
}

sub doNimNamedRequest
{
    my $address = shift;
    my $command = shift;
    my $data = shift;
    my $timeout = shift;
    my $retries = shift;

    ($rc, $pds_data);
    for (my $itry = 0; $itry < $retries; $itry++)
    {
#        print "nimNamedRequest addr: $address cmd: $command data: $data timeout: $timeout \n";
        ($rc, $pds_data) = nimNamedRequest($address, $command, $data, $timeout);
        last if (! $rc);
        if ($debug == 2) 
         {
           if ($command ne '_status' && $command ne 'list_subscribers' && $command ne 'getrobots' && $command ne 'probe_list')
            {
              print "rc=$rc for nimNamedRequest address=$address command=$command timeout=$timeout try=$itry\n";
            }
         }
    }
    return($rc, $pds_data);
}


sub getparams
{
  usage() unless @ARGV gt 0;
  my %Opts;
  getopts('d:f:r:i:a:q:o:b:s:t:g:v:',\%Opts);
  $outdir=$Opts{d};
  $outfile=$Opts{f};
  $action=$Opts{a};
  $verbose=$Opts{v};

  $filt_qos=$Opts{q};
  $filt_probe=$Opts{o};
  $filt_robot=$Opts{b};
  $filt_source=$Opts{s};
  $filt_target=$Opts{t};
  $filt_origin=$Opts{g};


# --- get credentials and settings from common file (nimsoft_generic.dat)
($Server_With_Rest_Interface, $RestServicePort, $UIM_username, $UIM_password,$UIM_Domain,$UIM_Hub,$UIM_Robot,$sql_server,$sql_user,$sql_password,$sql_db,$sql_type) = nimsoft_generic::getRestData();

$baseline_addr="/$UIM_Domain/$UIM_Hub/$UIM_Robot/baseline_engine";
# $baseline_addr="/$UIM_Domain/bgbulab00_hub/bgbulab00/baseline_engine";

# -- de-crypt (need $encoded)
$decoded = decode_base64($UIM_password);
$UIM_password = RC4($crkey, $decoded);

$decoded = decode_base64($sql_password);
$sql_password = RC4($crkey, $decoded);

# --- if not used in parameters, use defaults from generic
if ($sqluser eq '') { $sqluser=$sql_user; }
if ($sqlpass eq '') { $sqlpass=$sql_password; }
if ($sqlserver eq '') { $sqlserver=$sql_server; }
if ($database eq '') { $database=$sql_db; }

# --- set defaults
  if ($sqlopt eq '1' || $sqlopt eq '')
   {
      $sqltype="ms";
   }
  if ($sqlopt eq '2')
   {
      $sqltype="my";
   }
  if ($sql_type eq 'mssql')
   {
      $sqltype="ms";
   } 
  if ($sql_type eq 'mysql')
   {
      $sqltype="my";
   } 
  if ($sqluser eq '')
   {
     if ($sqltype eq 'my')
      {
        $sqluser = "root";
      }
     if ($sqltype eq 'ms')
      {
        if ($sqlpass eq '')
         {
           $sqluser="trusted";
         }
        else
         {
           $sqluser = "sa";
         }
      }
   }
  if ($sqluser eq 'trusted')
      {
         $trusted=yes;
      }
  if ($sqlpass eq '')
   {
     if ($sqluser eq 'trusted')
      {
         $trusted=yes;
      }
     else
      {
        print "By default we try to access the local mysql with user: $sqluser\n";
        print "BUT you need at least the -p parameter to give a password\n";
        exit 12;
      }
   }

  if ($database eq '')
   {
     $database="CA_UIM";
   }

  if ($sqlserver eq '')
   {
     $sqlserver=$hostn;
     if ($sqltype eq 'my')
      {
        print "We will connect the the mysql DB on server: $sqlserver\n";
      }
     if ($sqltype eq 'ms')
      {
        print "We will connect the the mssql DB on server: $sqlserver and userid: $sqluser\n";
      }
   }
# --- set defaults
  if ($outdir eq '')
   {
     $outdir = "c:\\temp";
   }

  if ($outfile eq '')
   {
     $outfile = "report\_nimsoft\_list\_baseline";
   }
  if ($action eq '')
   {
     $action="l";
   } 
  if ($action eq 'd' || $action eq 'k')
   {
     $doaction="delete";
   }
  if ($verbose eq '')
   {
     $verbose="n";
   } 
# --- create dir + name
$outfil = "$outdir\\$outfile";

%pds_type = (
	"PDS_I", 1,
	"PDS_PI", 2,
	"PDS_PPI", 3,
	"PDS_RGI", 4,
	"PDS_RGPI", 5,
	"PDS_CH", 6,
	"PDS_PCH", 7,
	"PDS_PPCH", 8,
	"PDS_RGCH", 9,
	"PDS_RGPCH", 10,
	"PDS_DUM_L", 11,
	"PDS_DUM_PL", 12,
	"PDS_DUM_PPL", 13,
	"PDS_DUM_RGL", 14,
	"PDS_DUM_RGPL", 15,
	"PDS_F", 16,
	"PDS_PF", 17,
	"PDS_PPF", 18,
	"PDS_RGF", 19,
	"PDS_RGPF", 20,
	"PDS_PDS", 21,
	"PDS_VOID", 22,
	"PDS_SEP", 23,
	"PDS_PPDS", 24,
	"PDS_CPCH", 25,
	"PDS_CPDS", 26,
	"PDS_INT64", 27
);

%pds_type2string = reverse %pds_type;
}

sub usage
 {
   print "      \n";
   print "nimsoft_list_baseline: list/delete baseline settings based on regex filters \n";
   print "   perl nimsoft_list_baseline.pl -a\"l\" -q\"CPU_USAGE\"\n";
   print "      \n";
   print "Input parameters:\n";
   print "  -d: output directory (default: c:\\temp)\n";
   print "  -f: output file (default: report_nimsoft_list_baseline.txt)\n";
#   print "  -u: sql userid (default: mysql: root mssql: sa)\n";
#   print "      if you use -u\"trusted\" we will use Windows trusted authentication\n";
#   print "  -p: sql password\n";
#   print "  -r: repository/server name: default local host\n";
#   print "  -y: 1: mssql 2: mysql (default: 1)\n";
#   print "  -8: database name, default: CA_UIM\n"; 
   print "Action:\n";
   print "  -a: Action l: list d: delete (default l)\n";
   print "  -v: verbose (y,n) default n. List non matching qos entries \n";
   print "Regex filters: (Case Insensitive)\n";
   print "  -q: qos name include regex \n";
   print "  -g: origin include regex\n";
   print "  -o: probe include regex \n";
   print "  -b: robot include regex \n";
   print "  -s: source include regex \n";
   print "  -t: target include regex \n";
   print "      \n";
   exit 0;
}
