#
# 1 - Read mssql db to find metid for qos entered as regex parameter (based on qos, origin, probe, robot, source and target)
# 2 - add/remove baseline java commands are generated in output file (that you must execute manually afterwards)
#
# By DEFAULT we will run in simulation mode so you can experiment with the regex filters until it's ok
# 
# 
# input parameters:
# -d: output directory (default: c:\temp)
# -f: output file (default: report_nimsoft_baseline)
#
# -k: y: delete n: simulate (default: n)
# -a: a: add d: delete
#
# --- these are possible regex matches (Case Insensitive)
# -q: qos name include regex 
# -g: oriGin include regex (Attention, this will/can delete a lot)
# -o: prObe include regex 
# -b: robot include regex 
# -s: source include regex 
# -t: target include regex 
#
# output:
# - print to output
#
# db: CA_UIM
#
# 30 December 2015 - Luc Christiaens
#
# Updates:
# 30/12/15: initial version
#         : nimsoft_generic.dat will only use passwords that are crypted via nimsoft_crypt.pl
# 05/02/16: if uim_user is set to trusted we will use Windows Authentication (UIM_password must be blank in that case)
# 16/03/16: send command to hub that is referenced as origin in qos, but for that we need to create a list of hubs and their robot and check if an alarm_enrichment probe is installed?
#

# --- Import external modules
use DBI;
use DBD::ODBC;
use Getopt::Std;
# crypt
use Crypt::RC4;
use MIME::Base64;
# UIM generic package
use nimsoft_generic;
# - change directory to directory where perl source is located to find our nimsoft_generic.dat file
use Cwd 'chdir';
use File::Basename;
chdir dirname $0;

# --- variables ---
$SERVER=$ENV{"computername"};
@hostnsplit=split(/\\/, $SERVER);
$hostnn=$#hostnsplit;
$hostn = $hostnsplit[$hostnn];
# - crypt key
$crkey       = "ab_def%hij&5";

# --- get parameters
&getparams();

# --- open logging file 
open(OUTFILE1, ">$outfil.txt") || die("Cannot open file $outfil : $!");

# --- loop variables
$sqlcount=1;
$teller=0;
$tel=0;
$totcol=0;
$qoscount=0;
$first="y";

# --- prepare date
&calcdate;

# --- connect DB ---
if ($sqltype eq 'my')
  {
    $dbh = DBI->connect("DBI:mysql:database=$database;host=$sqlserver", "$sqluser", "$sqlpass") || die "Could not connect to nimsoftslm database on $sqlserver: $DBI::errstr";
  }
if ($sqltype eq 'ms')
  {
    if ($trusted eq 'yes')
      {
        $dbh = DBI->connect("DBI:ODBC:Driver={SQL Server};Server=$sqlserver;Database=$database;Trusted_Connection=Yes;") or die("\n\nCONNECT ERROR:\n\n$DBI::errstr");
      }
    else
      {
        $dbh = DBI->connect("DBI:ODBC:Driver={SQL Server};Server=$sqlserver;Database=$database;UID=$sqluser;PWD=$sqlpass") or die("\n\nCONNECT ERROR:\n\n$DBI::errstr");
      }
  }



# --- sql query (in this case mysql and mssql are the same)
$sql = "SELECT qd.source, qd.qos, qd.target, qd.r_table, qd.table_id, qd.ci_metric_id, qd.probe, qd.origin, ci.ci_name, cid.ci_type, cid.ci_description FROM cm_configuration_item_definition cid,cm_configuration_item_metric cim,s_qos_data qd,cm_configuration_item ci,cm_device d,cm_computer_system cs WHERE cs.cs_id = d.cs_id AND d.dev_id = ci.dev_id AND cim.ci_metric_id = qd.ci_metric_id AND ci.ci_type = cid.ci_type AND ci.ci_id = cim.ci_id ORDER BY cid.ci_description, qd.probe, qd.qos, qd.target, qd.source ";
$sth = $dbh->prepare($sql);
$sth->execute or die "SQL Error: $DBI::errstr\n";

print "Active regex filters:\n";
print "  - qos   : $filt_qos\n";
print "  - origin: $filt_origin\n"; 
print "  - probe : $filt_probe\n";
print "  - robot : $filt_robot\n";
print "  - source: $filt_source\n";
print "  - target: $filt_target\n";
print "\n";

$count9=1;

while (@row = $sth->fetchrow_array) 
 {
# - decode line
  ($qsource,$qqos,$qtarget,$qrtable,$qtableid,$qmetid,$qprobe,$qorigin,$qci,$qcitype,$cidescription) = @row;
  $domatch=0;
# - match on filters
#   $domatch: 0 => no matching defined
#           : 1 => a filter was matched
#           : 2 => a filter did not match 
# once a filter has a $domatch=2 it will not be evaluated by others
#
    if ($filt_qos ne '')
     {
       if ($qqos =~ /$filt_qos/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }
    if ($filt_origin ne '')
     {
       if ($qorigin =~ /$filt_origin/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }
    if ($filt_probe ne '')
     {
       if ($qprobe =~ /$filt_probe/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }

    if ($filt_robot ne '')
     {
       if ($qrobot =~ /$filt_robot/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }

    if ($filt_source ne '')
     {
       if ($qsource =~ /$filt_source/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }

    if ($filt_target ne '')
     {
       if ($qtarget =~ /$filt_target/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }

# - if there is a match add it in the table (we need at least 1 filter)
#    if ($domatch eq '1' || $domatch eq '0')
    if ($domatch eq '1')
     {

# - put it in a table
       if ($dodelete ne 'y')
        {
           print "$count9: $qorigin $qrobot $qprobe $qqos $qsource $qtarget $qmetid\n";
           print OUTFILE1 "Simulation $count9: $qorigin $qrobot $qprobe $qqos $qsource $qtarget $qmetid\n";
        } 
          

# - put it in a table
       $qos9[$count9]="$qorigin\$qrobot\$qprobe\$qqos\$qsource\$qtarget\$qmetid";
       $count9++;
     }
    $sqlcount++;
 }

# how many qos did we see ?
$count9=$count9-1;
print " \n";
print "Processed $sqlcount QoS definitions and selected $count9 records\n";

if ($dodelete ne 'y')
 {
   print "Stop due to simulation mode run\n";
   close (OUTFILE1);
   exit;
 }
 
print "Start generating commands for the matched records\n";

# - we have all info, process the table and print all info with extra query to get qos details
$count9=1; 
$wait_count=0; 
while ($count9 < @qos9)
  {
     @numb9 = split (/\/, $qos9[$count9]); # create all keywords
     $yorigin=$numb9[0];
     $yrobot=$numb9[1];
     $yprobe=$numb9[2];
     $yqos = "$numb9[3]";
     $ysource = "$numb9[4]";
     $ytarget = "$numb9[5]";
     $ymetid = "$numb9[6]";


# - Generate baseline command in file
     if ($first eq 'y')
      {
        $first="n";
        print OUTFILE1 "cd C:\\Program Files (x86)\\Nimsoft\\probes\\slm\\baseline_engine\n";
      } 
     print OUTFILE1 "rem -- qos: $yqos robot: $yrobot probe: $yprobe source: $ysource target: $ytarget \n";
     print OUTFILE1 "java -cp .;lib/* com.nimsoft.threshold.cmd.BaselineSetter -user $UIM_username -pwd $UIM_password -probe /$UIM_Domain/$UIM_Hub/$UIM_Robot/baseline_engine -o $doaction -id $ymetid\n";
     $count9++;
 }

# --- disconnect ---
$dbh->disconnect();

# --- close HTML
close (OUTFILE1);


print "\n";
print "Created report: $outfil.txt\n";
print "\n";
exit 0;

# --- procedures


sub calcdate
 {
  $timeval = time();
  @timelist = gmtime ($timeval);
  if (@timelist[6] eq '0') { $dag = "Sunday" }
  if (@timelist[6] eq '1') { $dag = "Monday" }
  if (@timelist[6] eq '2') { $dag = "Tuesday" }
  if (@timelist[6] eq '3') { $dag = "Wednesday" }
  if (@timelist[6] eq '4') { $dag = "Thursday" }
  if (@timelist[6] eq '5') { $dag = "Friday" }
  if (@timelist[6] eq '6') { $dag = "Saturday" }
  if (@timelist[4] eq '0') { $maand = "January" }
  if (@timelist[4] eq '1') { $maand = "February" }
  if (@timelist[4] eq '2') { $maand = "March" }
  if (@timelist[4] eq '3') { $maand = "April" }
  if (@timelist[4] eq '4') { $maand = "May" }
  if (@timelist[4] eq '5') { $maand = "June" }
  if (@timelist[4] eq '6') { $maand = "July" }
  if (@timelist[4] eq '7') { $maand = "Augustus" }
  if (@timelist[4] eq '8') { $maand = "September" }
  if (@timelist[4] eq '9') { $maand = "October" }
  if (@timelist[4] eq '10') { $maand = "November" }
  if (@timelist[4] eq '11') { $maand = "December" }
  $maandnum = @timelist[4] +1;
  if ($maandnum < 10) {$maandnum="0$maandnum";}
  $dagnum = @timelist[3] ;
  if ($dagnum < 10) {$dagnum="0$dagnum";}
  $jaar = @timelist[5] + 1900;
#
# get time 
#
  $intim1 = time();
  $intim2 = localtime($intim1);
  @intime = split(/ +/ , $intim2);
  $intim = $intime[3]; 

# - format date to be used to compare with last poll (yyyy-mm-dd)
  $todayx="$jaar-$maandnum-$dagnum";


  print ("Today its $dag $dagnum $maand ($maandnum) $jaar $intim \n");
 }


sub getparams
{
  usage() unless @ARGV gt 0;
  my %Opts;
  getopts('d:f:u:p:r:y:q:k:o:b:s:t:v:w:n:g:8:a:',\%Opts);
  $outdir=$Opts{d};
  $outfile=$Opts{f};
  $sqluser=$Opts{u};
  $sqlpass=$Opts{p};
  $sqlopt=$Opts{y};
  $sqlserver=$Opts{r};
  $filt_qos=$Opts{q};
  $filt_probe=$Opts{o};
  $filt_robot=$Opts{b};
  $filt_source=$Opts{s};
  $filt_target=$Opts{t};
  $filt_origin=$Opts{g};
  $dodelete=$Opts{k};
  $database=$Opts{8};
  $action=$Opts{a};

# --- get credentials and settings from common file (nimsoft_generic.dat)
($Server_With_Rest_Interface, $RestServicePort, $UIM_username, $UIM_password,$UIM_Domain,$UIM_Hub,$UIM_Robot,$sql_server,$sql_user,$sql_password,$sql_db,$sql_type) = nimsoft_generic::getRestData();

# -- de-crypt (need $encoded)
$decoded = decode_base64($UIM_password);
$UIM_password = RC4($crkey, $decoded);

$decoded = decode_base64($sql_password);
$sql_password = RC4($crkey, $decoded);

# --- if not used in parameters, use defaults from generic
if ($sqluser eq '') { $sqluser=$sql_user; }
if ($sqlpass eq '') { $sqlpass=$sql_password; }
if ($sqlserver eq '') { $sqlserver=$sql_server; }
if ($database eq '') { $database=$sql_db; }

# --- set defaults
  if ($sqlopt eq '1' || $sqlopt eq '')
   {
      $sqltype="ms";
   }
  if ($sqlopt eq '2')
   {
      $sqltype="my";
   }
  if ($sql_type eq 'mssql')
   {
      $sqltype="ms";
   } 
  if ($sql_type eq 'mysql')
   {
      $sqltype="my";
   } 
  if ($sqluser eq '')
   {
     if ($sqltype eq 'my')
      {
        $sqluser = "root";
      }
     if ($sqltype eq 'ms')
      {
        if ($sqlpass eq '')
         {
           $sqluser="trusted";
         }
        else
         {
           $sqluser = "sa";
         }
      }
   }
  if ($sqluser eq 'trusted')
      {
         $trusted=yes;
      }
  if ($sqlpass eq '')
   {
     if ($sqluser eq 'trusted')
      {
         $trusted=yes;
      }
     else
      {
        print "By default we try to access the local mysql with user: $sqluser\n";
        print "BUT you need at least the -p parameter to give a password\n";
        exit 12;
      }
   }

  if ($database eq '')
   {
     $database="CA_UIM";
   }

  if ($sqlserver eq '')
   {
     $sqlserver=$hostn;
     if ($sqltype eq 'my')
      {
        print "We will connect the the mysql DB on server: $sqlserver\n";
      }
     if ($sqltype eq 'ms')
      {
        print "We will connect the the mssql DB on server: $sqlserver and userid: $sqluser\n";
      }
   }
  if ($outdir eq '')
   {
     $outdir = "c:\\temp";
   }
  if ($outfile eq '')
   {
     $outfile = "report\_nimsoft\_set\_baseline";
   }

  if ($dodelete eq '')
   {
     $dodelete="n";
     print "Running in Simulation mode, nothing will be defined/modified\n";
   }

  if ($action eq '' || $action eq 'a')
   {
     $doaction="add";
   }
  if ($action eq 'd')
   {
     $doaction="delete";
   }

# --- create dir + name
$outfil = "$outdir\\$outfile";
}

sub usage
 {
   print "      \n";
   print "nimsoft_set_baseline: activate/delete baseline settings based on regex filters \n";
   print "   perl nimsoft_set_baseline.pl -k\"n\" -q\"CPU_USAGE\"\n";
   print "      \n";
   print "Input parameters:\n";
   print "  -d: output directory (default: c:\\temp)\n";
   print "  -f: output file (default: report_nimsoft_set_baseline.txt)\n";
#   print "  -u: sql userid (default: mysql: root mssql: sa)\n";
#   print "      if you use -u\"trusted\" we will use Windows trusted authentication\n";
#   print "  -p: sql password\n";
#   print "  -r: repository/server name: default local host\n";
#   print "  -y: 1: mssql 2: mysql (default: 1)\n";
#   print "  -8: database name, default: CA_UIM\n"; 
   print "Action:\n";
   print "  -k: y: generate command in file n: simulate (default: n)\n";
   print "  -a: Action a: add d: delete (default a)\n";
   print "Regex filters: (Case Insensitive)\n";
   print "  -q: qos name include regex \n";
   print "  -g: origin include regex\n";
   print "  -o: probe include regex \n";
   print "  -b: robot include regex \n";
   print "  -s: source include regex \n";
   print "  -t: target include regex \n";
   print "      \n";
   exit 0;
}

